@@ -44,12 +44,6 @@ from qutebrowser.commands import userscripts, cmdexc, cmdutils, runners
 from qutebrowser.config import config, configexc
 from qutebrowser.browser import (urlmarks, browsertab, inspector, navigate,
                                  webelem, downloads)
-try:
-    from qutebrowser.browser.webkit import mhtml
-except ImportError:
-    # Failing imports on QtWebEngine, only used in QtWebKit commands.
-    # FIXME:qtwebengine don't import this anymore at all
-    pass
 from qutebrowser.keyinput import modeman
 from qutebrowser.utils import (message, usertypes, log, qtutils, urlutils,
                                objreg, utils, typing)
@@ -349,6 +343,43 @@ class CommandDispatcher:
         if tab is not None:
             tab.stop()
 
+    def _print_preview(self, tab):
+        """Show a print preview."""
+        def print_callback(ok):
+            if not ok:
+                message.error("Printing failed!")
+
+        tab.printing.check_preview_support()
+        diag = QPrintPreviewDialog()
+        diag.setAttribute(Qt.WA_DeleteOnClose)
+        diag.setWindowFlags(diag.windowFlags() | Qt.WindowMaximizeButtonHint |
+                            Qt.WindowMinimizeButtonHint)
+        diag.paintRequested.connect(functools.partial(
+            tab.printing.to_printer, callback=print_callback))
+        diag.exec_()
+
+    def _print_pdf(self, tab, filename):
+        """Print to the given PDF file."""
+        tab.printing.check_pdf_support()
+        filename = os.path.expanduser(filename)
+        directory = os.path.dirname(filename)
+        if directory and not os.path.exists(directory):
+            os.mkdir(directory)
+        tab.printing.to_pdf(filename)
+        log.misc.debug("Print to file: {}".format(filename))
+
+    def _print(self, tab):
+        """Print with a QPrintDialog."""
+        def print_callback(ok):
+            """Called when printing finished."""
+            if not ok:
+                message.error("Printing failed!")
+            diag.deleteLater()
+
+        diag = QPrintDialog()
+        diag.open(lambda: tab.printing.to_printer(diag.printer(),
+                                                  print_callback))
+
     @cmdutils.register(instance='command-dispatcher', name='print',
                        scope='window')
     @cmdutils.argument('count', count=True)
@@ -370,28 +401,17 @@ class CommandDispatcher:
                 tab.printing.check_pdf_support()
             else:
                 tab.printing.check_printer_support()
+            if preview:
+                tab.printing.check_preview_support()
         except browsertab.WebTabError as e:
             raise cmdexc.CommandError(e)
 
         if preview:
-            diag = QPrintPreviewDialog()
-            diag.setAttribute(Qt.WA_DeleteOnClose)
-            diag.setWindowFlags(diag.windowFlags() |
-                                Qt.WindowMaximizeButtonHint |
-                                Qt.WindowMinimizeButtonHint)
-            diag.paintRequested.connect(tab.printing.to_printer)
-            diag.exec_()
+            self._print_preview(tab)
         elif pdf:
-            pdf = os.path.expanduser(pdf)
-            directory = os.path.dirname(pdf)
-            if directory and not os.path.exists(directory):
-                os.mkdir(directory)
-            tab.printing.to_pdf(pdf)
-            log.misc.debug("Print to file: {}".format(pdf))
+            self._print_pdf(tab, pdf)
         else:
-            diag = QPrintDialog()
-            diag.setAttribute(Qt.WA_DeleteOnClose)
-            diag.open(lambda: tab.printing.to_printer(diag.printer()))
+            self._print(tab)
 
     @cmdutils.register(instance='command-dispatcher', scope='window')
     def tab_clone(self, bg=False, window=False):
@@ -407,6 +427,11 @@ class CommandDispatcher:
         cmdutils.check_exclusive((bg, window), 'bw')
         curtab = self._current_widget()
         cur_title = self._tabbed_browser.page_title(self._current_index())
+        try:
+            history = curtab.history.serialize()
+        except browsertab.WebTabError as e:
+            raise cmdexc.CommandError(e)
+
         # The new tab could be in a new tabbed_browser (e.g. because of
         # tabs-are-windows being set)
         if window:
@@ -417,13 +442,15 @@ class CommandDispatcher:
         new_tabbed_browser = objreg.get('tabbed-browser', scope='window',
                                         window=newtab.win_id)
         idx = new_tabbed_browser.indexOf(newtab)
+
         new_tabbed_browser.set_page_title(idx, cur_title)
         if config.get('tabs', 'show-favicons'):
             new_tabbed_browser.setTabIcon(idx, curtab.icon())
             if config.get('tabs', 'tabs-are-windows'):
                 new_tabbed_browser.window().setWindowIcon(curtab.icon())
+
         newtab.data.keep_icon = True
-        newtab.history.deserialize(curtab.history.serialize())
+        newtab.history.deserialize(history)
         newtab.zoom.set_factor(curtab.zoom.factor())
         return newtab
 
@@ -652,7 +679,7 @@ class CommandDispatcher:
         """
         tab = self._current_widget()
         if not tab.url().isValid():
-            # See https://github.com/The-Compiler/qutebrowser/issues/701
+            # See https://github.com/qutebrowser/qutebrowser/issues/701
             return
 
         if bottom_navigate is not None and tab.scroller.at_bottom():
@@ -755,7 +782,7 @@ class CommandDispatcher:
             perc = tab.zoom.offset(count)
         except ValueError as e:
             raise cmdexc.CommandError(e)
-        message.info("Zoom level: {}%".format(perc))
+        message.info("Zoom level: {}%".format(perc), replace=True)
 
     @cmdutils.register(instance='command-dispatcher', scope='window')
     @cmdutils.argument('count', count=True)
@@ -770,7 +797,7 @@ class CommandDispatcher:
             perc = tab.zoom.offset(-count)
         except ValueError as e:
             raise cmdexc.CommandError(e)
-        message.info("Zoom level: {}%".format(perc))
+        message.info("Zoom level: {}%".format(perc), replace=True)
 
     @cmdutils.register(instance='command-dispatcher', scope='window')
     @cmdutils.argument('count', count=True)
@@ -794,7 +821,7 @@ class CommandDispatcher:
             tab.zoom.set_factor(float(level) / 100)
         except ValueError:
             raise cmdexc.CommandError("Can't zoom {}%!".format(level))
-        message.info("Zoom level: {}%".format(level))
+        message.info("Zoom level: {}%".format(level), replace=True)
 
     @cmdutils.register(instance='command-dispatcher', scope='window')
     def tab_only(self, prev=False, next_=False):
@@ -833,7 +860,7 @@ class CommandDispatcher:
         """
         if self._count() == 0:
             # Running :tab-prev after last tab was closed
-            # See https://github.com/The-Compiler/qutebrowser/issues/1448
+            # See https://github.com/qutebrowser/qutebrowser/issues/1448
             return
         newidx = self._current_index() - count
         if newidx >= 0:
@@ -853,7 +880,7 @@ class CommandDispatcher:
         """
         if self._count() == 0:
             # Running :tab-next after last tab was closed
-            # See https://github.com/The-Compiler/qutebrowser/issues/1448
+            # See https://github.com/qutebrowser/qutebrowser/issues/1448
             return
         newidx = self._current_index() + count
         if newidx < self._count():
@@ -1302,58 +1329,39 @@ class CommandDispatcher:
         # FIXME:qtwebengine do this with the QtWebEngine download manager?
         download_manager = objreg.get('qtnetwork-download-manager',
                                       scope='window', window=self._win_id)
+        target = None
+        if dest is not None:
+            target = downloads.FileDownloadTarget(dest)
+
+        tab = self._current_widget()
+        user_agent = tab.user_agent()
+
         if url:
             if mhtml_:
                 raise cmdexc.CommandError("Can only download the current page"
                                           " as mhtml.")
             url = urlutils.qurl_from_user_input(url)
             urlutils.raise_cmdexc_if_invalid(url)
-            if dest is None:
-                target = None
-            else:
-                target = downloads.FileDownloadTarget(dest)
-            download_manager.get(url, target=target)
+            download_manager.get(url, user_agent=user_agent, target=target)
         elif mhtml_:
-            self._download_mhtml(dest)
-        else:
-            qnam = self._current_widget().networkaccessmanager()
-
-            if dest is None:
-                target = None
+            tab = self._current_widget()
+            if tab.backend == usertypes.Backend.QtWebEngine:
+                webengine_download_manager = objreg.get(
+                    'webengine-download-manager')
+                try:
+                    webengine_download_manager.get_mhtml(tab, target)
+                except browsertab.UnsupportedOperationError as e:
+                    raise cmdexc.CommandError(e)
             else:
-                target = downloads.FileDownloadTarget(dest)
-            download_manager.get(self._current_url(), qnam=qnam, target=target)
-
-    def _download_mhtml(self, dest=None):
-        """Download the current page as an MHTML file, including all assets.
-
-        Args:
-            dest: The file path to write the download to.
-        """
-        tab = self._current_widget()
-        if tab.backend == usertypes.Backend.QtWebEngine:
-            raise cmdexc.CommandError("Download --mhtml is not implemented "
-                                      "with QtWebEngine yet")
-
-        if dest is None:
-            suggested_fn = self._current_title() + ".mht"
-            suggested_fn = utils.sanitize_filename(suggested_fn)
-
-            filename = downloads.immediate_download_path()
-            if filename is not None:
-                mhtml.start_download_checked(filename, tab=tab)
-            else:
-                question = downloads.get_filename_question(
-                    suggested_filename=suggested_fn, url=tab.url(), parent=tab)
-                question.answered.connect(functools.partial(
-                    mhtml.start_download_checked, tab=tab))
-                message.global_bridge.ask(question, blocking=False)
+                download_manager.get_mhtml(tab, target)
         else:
-            mhtml.start_download_checked(dest, tab=tab)
+            qnam = tab.networkaccessmanager()
+            download_manager.get(self._current_url(), user_agent=user_agent,
+                                 qnam=qnam, target=target)
 
     @cmdutils.register(instance='command-dispatcher', scope='window')
     def view_source(self):
-        """Show the source of the current page."""
+        """Show the source of the current page in a new tab."""
         # pylint: disable=no-member
         # WORKAROUND for https://bitbucket.org/logilab/pylint/issue/491/
         tab = self._current_widget()
@@ -1400,6 +1408,18 @@ class CommandDispatcher:
 
         tab.dump_async(callback, plain=plain)
 
+    @cmdutils.register(instance='command-dispatcher', scope='window')
+    def history(self, tab=True, bg=False, window=False):
+        """Show browsing history.
+
+        Args:
+            tab: Open in a new tab.
+            bg: Open in a background tab.
+            window: Open in a new window.
+        """
+        url = QUrl('qute://history/')
+        self._open(url, tab, bg, window)
+
     @cmdutils.register(instance='command-dispatcher', name='help',
                        scope='window')
     @cmdutils.argument('topic', completion=usertypes.Completion.helptopic)
@@ -1545,7 +1565,8 @@ class CommandDispatcher:
     @cmdutils.argument('filter_', choices=['id'])
     def click_element(self, filter_: str, value, *,
                       target: usertypes.ClickTarget=
-                      usertypes.ClickTarget.normal):
+                      usertypes.ClickTarget.normal,
+                      force_event=False):
         """Click the element matching the given filter.
 
         The given filter needs to result in exactly one element, otherwise, an
@@ -1556,6 +1577,7 @@ class CommandDispatcher:
                      id: Get an element based on its ID.
             value: The value to filter for.
             target: How to open the clicked element (normal/tab/tab-bg/window).
+            force_event: Force generating a fake click event.
         """
         tab = self._current_widget()
 
@@ -1565,7 +1587,7 @@ class CommandDispatcher:
                 message.error("No element found with id {}!".format(value))
                 return
             try:
-                elem.click(target)
+                elem.click(target, force_event=force_event)
             except webelem.Error as e:
                 message.error(str(e))
                 return
@@ -1905,12 +1927,13 @@ class CommandDispatcher:
 
     @cmdutils.register(instance='command-dispatcher', scope='window',
                        maxsplit=0, no_cmd_split=True)
-    def jseval(self, js_code, quiet=False, *,
+    def jseval(self, js_code, file=False, quiet=False, *,
                world: typing.Union[usertypes.JsWorld, int]=None):
         """Evaluate a JavaScript string.
 
         Args:
-            js_code: The string to evaluate.
+            js_code: The string/file to evaluate.
+            file: Interpret js-code as a path to a file.
             quiet: Don't show resulting JS object.
             world: Ignored on QtWebKit. On QtWebEngine, a world ID or name to
                    run the snippet in.
@@ -1938,6 +1961,13 @@ class CommandDispatcher:
                         out = out[:5000] + ' [...trimmed...]'
                     message.info(out)
 
+        if file:
+            try:
+                with open(js_code, 'r', encoding='utf-8') as f:
+                    js_code = f.read()
+            except OSError as e:
+                raise cmdexc.CommandError(str(e))
+
         widget = self._current_widget()
         widget.run_js_async(js_code, callback=jseval_cb, world=world)
 
@@ -2045,3 +2075,24 @@ class CommandDispatcher:
         """
         if bg or tab or window or url != old_url:
             self.openurl(url=url, bg=bg, tab=tab, window=window)
+
+    @cmdutils.register(instance='command-dispatcher', scope='window')
+    def fullscreen(self, leave=False):
+        """Toggle fullscreen mode.
+
+        Args:
+            leave: Only leave fullscreen if it was entered by the page.
+        """
+        if leave:
+            tab = self._current_widget()
+            try:
+                tab.action.exit_fullscreen()
+            except browsertab.UnsupportedOperationError:
+                pass
+            return
+
+        window = self._tabbed_browser.window()
+        if window.isFullScreen():
+            window.showNormal()
+        else:
+            window.showFullScreen()
