@@ -32,7 +32,7 @@ from PyQt5.QtWebEngineWidgets import (QWebEngineSettings, QWebEngineProfile,
 # pylint: enable=no-name-in-module,import-error,useless-suppression
 
 from qutebrowser.browser import shared
-from qutebrowser.config import websettings
+from qutebrowser.config import config, websettings
 from qutebrowser.utils import objreg, utils, standarddir, javascript
 
 
@@ -65,6 +65,45 @@ class StaticSetter(websettings.StaticSetter):
     GLOBAL_SETTINGS = QWebEngineSettings.globalSettings
 
 
+class ProfileSetter(websettings.Base):
+
+    """A setting set on the QWebEngineProfile."""
+
+    def __init__(self, getter, setter):
+        super().__init__()
+        profile = QWebEngineProfile.defaultProfile()
+        self._getter = getattr(profile, getter)
+        self._setter = getattr(profile, setter)
+
+    def get(self, settings=None):
+        utils.unused(settings)
+        return self._getter()
+
+    def _set(self, value, settings=None):
+        utils.unused(settings)
+        self._setter(value)
+
+
+class PersistentCookiePolicy(ProfileSetter):
+
+    """The cookies -> store setting is different from other settings."""
+
+    def __init__(self):
+        super().__init__(getter='persistentCookiesPolicy',
+                         setter='setPersistentCookiesPolicy')
+
+    def get(self, settings=None):
+        utils.unused(settings)
+        return config.get('content', 'cookies-store')
+
+    def _set(self, value, settings=None):
+        utils.unused(settings)
+        self._setter(
+            QWebEngineProfile.AllowPersistentCookies if value else
+            QWebEngineProfile.NoPersistentCookies
+        )
+
+
 def _init_stylesheet(profile):
     """Initialize custom stylesheets.
 
@@ -98,6 +137,13 @@ def _init_stylesheet(profile):
     profile.scripts().insert(script)
 
 
+def _init_profile(profile):
+    """Initialize settings set on the QWebEngineProfile."""
+    profile.setCachePath(os.path.join(standarddir.cache(), 'webengine'))
+    profile.setPersistentStoragePath(
+        os.path.join(standarddir.data(), 'webengine'))
+
+
 def update_settings(section, option):
     """Update global settings when qwebsettings changed."""
     websettings.update_mappings(MAPPINGS, section, option)
@@ -112,10 +158,13 @@ def init(args):
         os.environ['QTWEBENGINE_REMOTE_DEBUGGING'] = str(utils.random_port())
 
     profile = QWebEngineProfile.defaultProfile()
-    profile.setCachePath(os.path.join(standarddir.cache(), 'webengine'))
-    profile.setPersistentStoragePath(
-        os.path.join(standarddir.data(), 'webengine'))
+    _init_profile(profile)
     _init_stylesheet(profile)
+    # We need to do this here as a WORKAROUND for
+    # https://bugreports.qt.io/browse/QTBUG-58650
+    PersistentCookiePolicy().set(config.get('content', 'cookies-store'))
+
+    Attribute(QWebEngineSettings.FullScreenSupportEnabled).set(True)
 
     websettings.init_mappings(MAPPINGS)
     objreg.get('config').changed.connect(update_settings)
@@ -127,24 +176,17 @@ def shutdown():
 
 
 # Missing QtWebEngine attributes:
-# - ErrorPageEnabled (should not be exposed, but set)
-# - FullScreenSupportEnabled
 # - ScreenCaptureEnabled
 # - Accelerated2dCanvasEnabled
 # - AutoLoadIconsForPage
 # - TouchIconsEnabled
+# - FocusOnNavigationEnabled (5.8)
+# - AllowRunningInsecureContent (5.8)
 #
 # Missing QtWebEngine fonts:
 # - FantasyFont
 # - PictographFont
-#
-# TODO settings on profile:
-# - httpCacheMaximumSize
-# - persistentCookiesPolicy
-# - offTheRecord
-#
-# TODO settings elsewhere:
-# - proxy
+
 
 MAPPINGS = {
     'content': {
@@ -164,6 +206,11 @@ MAPPINGS = {
             Attribute(QWebEngineSettings.LocalContentCanAccessRemoteUrls),
         'local-content-can-access-file-urls':
             Attribute(QWebEngineSettings.LocalContentCanAccessFileUrls),
+        # https://bugreports.qt.io/browse/QTBUG-58650
+        # 'cookies-store':
+        #     PersistentCookiePolicy(),
+        'webgl':
+            Attribute(QWebEngineSettings.WebGLEnabled),
     },
     'input': {
         'spatial-navigation':
@@ -220,6 +267,9 @@ MAPPINGS = {
     'storage': {
         'local-storage':
             Attribute(QWebEngineSettings.LocalStorageEnabled),
+        'cache-size':
+            ProfileSetter(getter='httpCacheMaximumSize',
+                          setter='setHttpCacheMaximumSize')
     },
     'general': {
         'xss-auditing':
@@ -231,7 +281,8 @@ MAPPINGS = {
 }
 
 try:
-    MAPPINGS['content']['webgl'] = Attribute(QWebEngineSettings.WebGLEnabled)
+    MAPPINGS['general']['print-element-backgrounds'] = Attribute(
+        QWebEngineSettings.PrintElementBackgrounds)
 except AttributeError:
-    # Added in Qt 5.7
+    # Added in Qt 5.8
     pass
